
#######################################################
#
# Results from nonparametric bootstrapping procedure
# for predictions, Table 5
#
#######################################################

# clear memory
rm(list=ls()) 

# set required parameters
#############################

# drawn data set size
N <- 1000

# number of monte carlo repetitions
reps <- 1000

# the set of correlations to examine
R<-c(0.99, 0.9, 0.5, 0.2, 0, -0.2, -0.5, -0.9, -0.99)





# load required libraries
require(mvtnorm)
require(cubature)
require(boot)
require(snow)
require(rlecuyer)


# using matrices of bootstrapped samples of calculated marginal
# effects, determine the limits for the t-statistic that correspond to a 
# rejection rate of err 

find.multi.lims<-function(mat1, mat2=NULL, mat3=NULL, mat4=NULL, p1=99, p2=NULL, p3=NULL, p4=NULL, type="any", err=0.05){
  
  # bootstrap replicates of marginal effects in the rows of matK
  # points of evaluation in the columns of matK
  
  # pK: type of hypothesis test for the MEs in matK
  # 1 = one-sided test for ME > 0
  # -1 = one-sided test for ME < 0
  # 0 = null of ME = 0 cannot be rejected
  # any other value = two-sided test for ME != 0
  
  # type: the condition that corresponds to successful rejection of the null
  # "any" = any of the matK marginal effects is statistically significant in the direction given by pK
  # "all" = ALL of the matK marginal effects are statistically significant and in the direction given by pK
  
  
  # a function to determine the number of one-tailed statistically significant
  # estimates in a vector v1 given upper limits v2 and lower limits v3
  sig<-function(v1, v2, v3, p){
    
    a<-max(v1>v2)
    b<-max(v1<v3)
    if(p==0){return(abs(max(a,b)-1))}
    if(p==1){return(a)}
    if(p==-1){return(b)}
    return(max(a,b))
    
  }
  
  # a function to return a squared error term corresponding to how far the false
  # positive rate produced by t quantile limits (K, 1-K) is from 5%
  lims<-function(K, sum.type=type){
    
    lim.cand<-c(-K, K)
    
    m2<-NA; m3<-NA; m4<-NA
    
    m1<-apply(X=mat1, FUN=sig, MARGIN=1, v2=lim.cand[2], v3=lim.cand[1], p1)
    n1<-nrow(mat1)
    if(is.null(mat2)==FALSE){m2<-apply(X=mat2, FUN=sig, MARGIN=1, v2=lim.cand[2], v3=lim.cand[1], p2)}
    if(is.null(mat3)==FALSE){m3<-apply(X=mat3, FUN=sig, MARGIN=1, v2=lim.cand[2], v3=lim.cand[1], p3)}
    if(is.null(mat4)==FALSE){m4<-apply(X=mat4, FUN=sig, MARGIN=1, v2=lim.cand[2], v3=lim.cand[1], p4)}
    
    if(sum.type=="any"){optimand<-sum(pmax(m1, m2, m3, m4, na.rm=T))/sum(n1)}
    if(sum.type=="all"){optimand<-sum(pmin(m1, m2, m3, m4, na.rm=T))/sum(n1)}
    
    return((optimand-err)^2)
  }
  
  # find the t quantile limits that get as close as possible to a 5% rejection rate
  # in the bootstrap sample at hand
  
  if(type=="any"){out<-optimize(f=lims, interval=c(0.001, 5), sum.type=type)}
  if(type=="all"){out<-optimize(f=lims, interval=c(0.001, 2), sum.type=type)}
  
  # if there were no hits at all, just return t=0 as the optimum
  if(err^2==out$objective){out<-list(minimum=0, objective=NA); return(out)}else{return(out)}
  
}


# function used to create non-parametrically bootstrapped samples of marginal effects
# calculated from a model with interaction; takes data, model, an index of observations, 
# and identities of the interacted variables and returns a vector of t-statistics
# corresponding to a single bootstrap deviate (of observations indexed in K) for 
# marginal effects in the direction of x.name and z.name
boot.fun<-function(dat, K, form, fam="gaussian", x.name, z.name){
  
  # dat: the data set on which the original model was run
  # K: index of observations in dat that will be selected for a particular bootstrap run
  # form: the formula for the GLM model
  # fam: the family for the GLM model
  # x.name: the identity of the first interacted variable
  # z.name: the identity of the second interacted variable
  
  # note: form should include x.name*z.name, in that order.
  
  # remove all extraneous variables from the data, delete missing observations
  dat<-subset(dat, select=all.vars(form))
  dat<-na.omit(dat)
  
  # anticipate what the name of the interaction term will be
  int.name<-paste(x.name, ":", z.name, sep="")
  
  # fit the model to the full data set
  int.mod<-glm(formula=form, family=fam, data=dat)
  
  # determine what MEs to calculate in the x dimension
  # calculate at every unique value of z, OR ten evenly spaced values of z, depending on the nature of z
  if(length(unique(dat[[z.name]]))<=10)
  {z.test<-sort(unique(dat[[z.name]]))}else{z.test<-seq(from=min(dat[[z.name]]), to=max(dat[[z.name]]), length.out=10)}
  # calculate dy/dx | z for the values of z.test
  coef.x.o<-summary(int.mod)$coefficients[x.name,1]+summary(int.mod)$coefficients[int.name,1]*z.test
  
  # determine what MEs to calculate in the z dimension
  # calculate at every unique value of x, OR ten evenly spaced values of x, depending on the nature of x
  if(length(unique(dat[[x.name]]))<=10)
  {x.test<-sort(unique(dat[[x.name]]))}else{x.test<-seq(from=min(dat[[x.name]]), to=max(dat[[x.name]]), length.out=10)}
  # calculate dy/dz | x for the values of z.test
  coef.z.o<-summary(int.mod)$coefficients[z.name,1]+summary(int.mod)$coefficients[int.name,1]*x.test
  
  # create the bootstrapped data set using the indices K
  dat.boot<-dat[K,]
  
  # run the model on the bootstrapped data
  int.mod.store<-glm(formula=form, family=fam, data=dat.boot)
  
  # calculate dy/dx | z for the values of z.test
  coef.x<-summary(int.mod.store)$coefficients[x.name,1]+summary(int.mod.store)$coefficients[int.name,1]*z.test
  se.x<-sqrt(vcov(int.mod.store)[x.name,x.name]+(z.test^2)*vcov(int.mod.store)[int.name,int.name]+2*z.test*vcov(int.mod.store)[x.name,int.name])
  
  # calculate the bootstrap-t values for dy/dx | z
  t.stat.x<-(coef.x-coef.x.o)/se.x
  
  # calculate dy/dz | x for the values of z.test
  coef.z<-summary(int.mod.store)$coefficients[z.name,1]+summary(int.mod.store)$coefficients[int.name,1]*x.test
  se.z<-sqrt(vcov(int.mod.store)[z.name,z.name]+(x.test^2)*vcov(int.mod.store)[int.name,int.name]+2*x.test*vcov(int.mod.store)[z.name,int.name])
  
  # calculate the bootstrap-t values for dy/dz | x
  t.stat.z<-(coef.z-coef.z.o)/se.z
  
  # return the bootstrap-t statistics from the function
  return(c(t.stat.x, t.stat.z))
  
}




# initialize base random number generator
set.seed(024823)

# start cluster and initialize RNG on the cluster
cl<-makeCluster(20, type="SOCK")
clusterSetupRNGstream(cl, seed=rep(12345,6))

# storage for critical t statistics
single.me.five<-matrix(data=NA, nrow=reps, ncol=length(R))
single.me.ten<-matrix(data=NA, nrow=reps, ncol=length(R))
double.me.five<-matrix(data=NA, nrow=reps, ncol=length(R))
double.me.ten<-matrix(data=NA, nrow=reps, ncol=length(R))
one.insig.one.dir<-matrix(data=NA, nrow=reps, ncol=length(R))
opposite.sign<-matrix(data=NA, nrow=reps, ncol=length(R))
opposite.sign.both<-matrix(data=NA, nrow=reps, ncol=length(R))
opposite.sign.both.dir.one<-matrix(data=NA, nrow=reps, ncol=length(R))
opposite.sign.both.other.way<-matrix(data=NA, nrow=reps, ncol=length(R))

# storage for significance frequencies
me.five.sig <- matrix(data=NA, nrow=reps, ncol=length(R))
me.ten.sig <- matrix(data=NA, nrow=reps, ncol=length(R))
double.five.sig <- matrix(data=NA, nrow=reps, ncol=length(R))
double.ten.sig <- matrix(data=NA, nrow=reps, ncol=length(R))
one.dir.sig <- matrix(data=NA, nrow=reps, ncol=length(R))
opposite.sig <- matrix(data=NA, nrow=reps, ncol=length(R))
opposite.both.sig <- matrix(data=NA, nrow=reps, ncol=length(R))
opposite.both.dir.sig <- matrix(data=NA, nrow=reps, ncol=length(R))
opposite.both.other.sig <- matrix(data=NA, nrow=reps, ncol=length(R))

# cycle through all the correlations and calculate critical t
for(j in 1:length(R)){
  
  cat("Calculating for covariance = ", R[j], "...\n", sep="")
  # establish the covariance of the interacted variables, x and z
  SIGMA <- matrix(data=c(1, R[j], R[j], 1), nrow=2, ncol=2)
  
  pb<-txtProgressBar(initial=0, min=1, max=reps, style=3)
  
  for(i in 1:reps){
    
    setTxtProgressBar(pb, value=i)
    
    # draw a data set of x and z from a common distribution
    X<-rmvnorm(N, mean=c(0,0), sigma=SIGMA)
    x<-X[,1]
    z<-X[,2]
    #x<-runif(N)
    #z<-runif(N)
    
    # compute y as a function of x and z (where both are null, in this case)
    y<-rnorm(N, mean=0, sd=1)
    
    # run interaction model and store it
    int.mod.store<-lm(y~x*z)
    
    x.test<-seq(from=-3, to=3, length.out=10)
    z.test<-seq(from=-3, to=3, length.out=10)
    
    # compute interaction effects
    coef.x<-summary(int.mod.store)$coefficients[2,1]+summary(int.mod.store)$coefficients[4,1]*z.test
    se.x<-sqrt(vcov(int.mod.store)[2,2]+(z.test^2)*vcov(int.mod.store)[4,4]+2*z.test*vcov(int.mod.store)[2,4])
    
    coef.z<-summary(int.mod.store)$coefficients[3,1]+summary(int.mod.store)$coefficients[4,1]*x.test
    se.z<-sqrt(vcov(int.mod.store)[3,3]+(x.test^2)*vcov(int.mod.store)[4,4]+2*x.test*vcov(int.mod.store)[3,4])
    
    # compute t-statistics from interactions
    t.stat.x<-coef.x/se.x
    t.stat.z<-coef.z/se.z
    
    
    # bootstrap the distribution of the MEs of X and Z based on the drawn data
    boot.t.dist<-boot(data=data.frame(y,x,z), statistic=boot.fun, R=10000, form=y~x*z, x.name="x", z.name="z", parallel = ("snow"), ncpus = 20, cl=cl)$t
    
    # sort the bootstrapped data into sub-components
    boot.t.x.dist<-boot.t.dist[,1:10]       # bootstrapped dy/dx at all z
    boot.t.z.dist<-boot.t.dist[,11:20]      # bootstrapped dy/dz at all x
    boot.t.x.dist.lo<-boot.t.dist[,1:5]     # bootstrapped dy/dx at lowest 5 values of z
    boot.t.x.dist.hi<-boot.t.dist[,6:10]    # bootstrapped dy/dx at highest 5 values of z
    boot.t.z.dist.lo<-boot.t.dist[,11:15]   # bootstrapped dy/dz at lowest 5 values of x
    boot.t.z.dist.hi<-boot.t.dist[,15:20]   # bootstrapped dy/dz at highest 5 values of x
    
    # critical t for whether x is statistically significant in either direction
    # total error rate = 0.05
    single.me.five[i,j]<-find.multi.lims(boot.t.x.dist, type="any", err=0.05)$minimum
    # total error rate = 0.1
    single.me.ten[i,j]<-find.multi.lims(boot.t.x.dist, type="any", err=0.10)$minimum
    
    # critical t for whether x OR z is statistically significant in either direction
    # total error rate = 0.05
    double.me.five[i,j]<-find.multi.lims(boot.t.x.dist, boot.t.z.dist, p1=99, p2=99, type="any", err=0.05)$minimum
    # total error rate = 0.10
    double.me.ten[i,j]<-find.multi.lims(boot.t.x.dist, boot.t.z.dist, p1=99, p2=99, type="any", err=0.10)$minimum
    
    # critical t for whether (dy/dx < 0 | z hi), (dy/dx = 0 | z low), total error rate = 0.05
    one.insig.one.dir[i,j]<-find.multi.lims(boot.t.x.dist.hi, boot.t.x.dist.lo, p1=-1, p2=0, type="all", err=0.05)$minimum
    
    # critical t for whether (dy/dx < 0 | z hi), (dy/dx > 0 | z low), total error rate = 0.05
    opposite.sign[i,j]<-find.multi.lims(boot.t.x.dist.hi, boot.t.x.dist.lo, p1=-1, p2=1, type="all", err=0.05)$minimum
    
    # critical t for whether (dy/dx < 0 | z hi), (dy/dx > 0 | z low), (dy/dz < 0 | x hi), (dy/dz > 0 | x low), one-tailed alpha = 0.05
    opposite.sign.both[i,j]<-find.multi.lims(boot.t.x.dist.hi, boot.t.x.dist.lo, boot.t.z.dist.hi, boot.t.z.dist.lo, p1=-1, p2=1, p3=-1, p4=1, type="all")$minimum
    
    # critical t for whether (dy/dx < 0 | z hi), (dy/dx > 0 | z low), (dy/dz < 0 | x hi), (dy/dz > 0 | x low), one-tailed alpha = 0.05
    opposite.sign.both.dir.one[i,j]<-find.multi.lims(boot.t.x.dist.hi, boot.t.x.dist.lo, boot.t.z.dist, p1=-1, p2=1, p3=-1, type="all")$minimum
    
    # critical t for whether (dy/dx < 0 | z hi), (dy/dx > 0 | z low), (dy/dz > 0 | x hi), (dy/dz < 0 | x low), one-tailed alpha = 0.05
    opposite.sign.both.other.way[i,j]<-find.multi.lims(boot.t.x.dist.hi, boot.t.x.dist.lo, boot.t.z.dist.hi, boot.t.z.dist.lo, p1=-1, p2=1, p3=1, p4=-1, type="all")$minimum
    
    
    # is at least one dy/dx statistically significant?
    me.five.sig[i,j] <- max(as.numeric( abs(t.stat.x) > single.me.five[i,j] ))
    me.ten.sig[i,j] <- max(as.numeric( abs(t.stat.x) > single.me.ten[i,j] ))
    
    # is at least one dy/dx OR dy/dz statistically significant?
    double.five.sig[i,j] <- max( as.numeric(abs(t.stat.x) > double.me.five[i,j]), as.numeric(abs(t.stat.z) > double.me.five[i,j])  )
    double.ten.sig[i,j] <- max( as.numeric(abs(t.stat.x) > double.me.ten[i,j]), as.numeric(abs(t.stat.z) > double.me.ten[i,j])  )
    
    # is dy/dx statistically insignificant for small z, and significant + negative for large z?
    one.dir.sig[i,j] <- min( min(as.numeric( abs(t.stat.x[1:5]) < one.insig.one.dir[i,j])), max(as.numeric( t.stat.x[6:10] < (-1)*one.insig.one.dir[i,j]))  )
    
    # is dy/dx statistically signiciant + positive for small z, and significant + negative for large z?
    opposite.sig[i,j] <- min( max(as.numeric( t.stat.x[1:5] > opposite.sign[i,j])), max(as.numeric( t.stat.x[6:10] < (-1)*opposite.sign[i,j])) )
    
    # is dy/dx statistically significant + positive for small z, significant + negative for large z, and dy/dz statistically significant + positive for small x, and significant + negative for large x?
    opposite.both.sig[i,j] <- min( max(as.numeric( t.stat.x[1:5] > opposite.sign.both[i,j])), max(as.numeric( t.stat.x[6:10] < (-1)*opposite.sign.both[i,j])),  max(as.numeric( t.stat.z[1:5] > opposite.sign.both[i,j])), max(as.numeric( t.stat.z[6:10] < (-1)*opposite.sign.both[i,j])) ) 
    
    # is dy/dx statistically significant + positive for small z, significant + negative for large z, and dy/dz statistically significant + negative at some point?
    opposite.both.dir.sig[i,j] <- min( max(as.numeric( t.stat.x[1:5] > opposite.sign.both.dir.one[i,j])), max(as.numeric( t.stat.x[6:10] < (-1)*opposite.sign.both.dir.one[i,j])),  max(as.numeric( t.stat.z[1:10] < (-1)*opposite.sign.both.dir.one[i,j])) ) 
    
    # is dy/dx statistically significant + positive for small z, significant + negative for large z, and dy/dz statistically significant + negative for small x, and significant + positive for large x?
    opposite.both.other.sig[i,j] <- min( max(as.numeric( t.stat.x[1:5] > opposite.sign.both.other.way[i,j])), max(as.numeric( t.stat.x[6:10] < (-1)*opposite.sign.both.other.way[i,j])),  max(as.numeric( t.stat.z[1:5] < (-1)*opposite.sign.both.other.way[i,j])), max(as.numeric( t.stat.z[6:10] > opposite.sign.both.other.way[i,j])) ) 
    
    
  }
  close(pb)
  
}

stopCluster(cl)

save.image(file="2016-8-1_interact-bootstrap-cont-results-normal.RData")

# load(file="2016-8-1_interact-bootstrap-cont-results-normal.RData")

# show the critical t-statistics for discoveries and predictions
round(apply(X=single.me.five, FUN=quantile, MARGIN=2, probs=c(0.025, 0.5, 0.975)), 2)
round(apply(X=single.me.ten, FUN=quantile, MARGIN=2, probs=c(0.025, 0.5, 0.975)), 2)
round(apply(X=double.me.five, FUN=quantile, MARGIN=2, probs=c(0.025, 0.5, 0.975)), 2)
round(apply(X=double.me.ten, FUN=quantile, MARGIN=2, probs=c(0.025, 0.5, 0.975)), 2)
# this is table 5, row 1
round(apply(X=one.insig.one.dir, FUN=quantile, MARGIN=2, probs=c(0.025, 0.5, 0.975)), 2)
# this is table 5, row 3
round(apply(X=opposite.sign, FUN=quantile, MARGIN=2, probs=c(0.025, 0.5, 0.975)), 2)
# this is table 5, row 5
round(apply(X=opposite.sign.both, FUN=quantile, MARGIN=2, probs=c(0.025, 0.5, 0.975)), 2)
# this is table 5, row 7
round(apply(X=opposite.sign.both.dir.one, FUN=quantile, MARGIN=2, probs=c(0.025, 0.5, 0.975)), 2)
round(apply(X=opposite.sign.both.other.way, FUN=quantile, MARGIN=2, probs=c(0.025, 0.5, 0.975)), 2)


# show the rejection rates for the bootstrapping procedure
round(apply(X=me.five.sig, FUN=sum, MARGIN=2)/reps, 2)
round(apply(X=me.ten.sig, FUN=sum, MARGIN=2)/reps, 2)
round(apply(X=double.five.sig, FUN=sum, MARGIN=2)/reps, 2)
round(apply(X=double.ten.sig, FUN=sum, MARGIN=2)/reps, 2)
# this is table 5, row 1
round(apply(X=one.dir.sig, FUN=sum, MARGIN=2)/reps, 2)
# this is table 5, row 3
round(apply(X=opposite.sig, FUN=sum, MARGIN=2)/reps, 2)
# this is table 5, row 5
round(apply(X=opposite.both.sig, FUN=sum, MARGIN=2)/reps, 2)
# this is table 5, row 7
round(apply(X=opposite.both.dir.sig, FUN=sum, MARGIN=2)/reps, 2)
round(apply(X=opposite.both.other.sig, FUN=sum, MARGIN=2)/reps, 2)



