##############################################################
##############################################################
#
# Replication Script for Clark-Golder Reanalysis:
# Script to Calculate Non-parametrically Bootstrapped 
# Critical-t Statistics for Discovery and Prediction
#
# for 
#
# "Testing Interaction Hypotheses: Determining and Controlling
# the False Positive Rate"
# 
# by
# 
# Justin Esarey and Jane Lawrence Sumner
# 8/2/2016
#
# Note: legislative_figure.dta file is generated in
# "2014-8-20_presidentialfig_corrected_discovery_cps2.do"
# but also included in the replication package
#
# Plots using these critical-t statistics are created using
# 2016-8-2_ethnicfig_corrected_theory_cps2.do
#
#
##############################################################
##############################################################

rm(list=ls())
library(foreign)

# using matrices of bootstrapped samples of calculated marginal
# effects, determine the limits for the t-statistic that correspond to a 
# rejection rate of err 

find.multi.lims<-function(mat1, mat2=NULL, mat3=NULL, mat4=NULL, p1=99, p2=NULL, p3=NULL, p4=NULL, type="any", err=0.05){
  
  # bootstrap replicates of marginal effects in the rows of matK
  # points of evaluation in the columns of matK
  
  # pK: type of hypothesis test for the MEs in matK
  # 1 = one-sided test for ME > 0
  # -1 = one-sided test for ME < 0
  # 0 = null of ME = 0 cannot be rejected
  # any other value = two-sided test for ME != 0
  
  # type: the condition that corresponds to successful rejection of the null
  # "any" = any of the matK marginal effects is statistically significant in the direction given by pK
  # "all" = ALL of the matK marginal effects are statistically significant and in the direction given by pK
  
  
  # a function to determine the number of one-tailed statistically significant
  # estimates in a vector v1 given upper limits v2 and lower limits v3
  sig<-function(v1, v2, v3, p){
    
    a<-max(v1>v2)
    b<-max(v1<v3)
    if(p==0){return(abs(max(a,b)-1))}
    if(p==1){return(a)}
    if(p==-1){return(b)}
    return(max(a,b))
    
  }
  
  # a function to return a squared error term corresponding to how far the false
  # positive rate produced by t quantile limits (K, 1-K) is from 5%
  lims<-function(K, sum.type=type){
    
    lim.cand<-c(-K, K)
    
    m2<-NA; m3<-NA; m4<-NA
    
    m1<-apply(X=mat1, FUN=sig, MARGIN=1, v2=lim.cand[2], v3=lim.cand[1], p1)
    n1<-nrow(mat1)
    if(is.null(mat2)==FALSE){m2<-apply(X=mat2, FUN=sig, MARGIN=1, v2=lim.cand[2], v3=lim.cand[1], p2)}
    if(is.null(mat3)==FALSE){m3<-apply(X=mat3, FUN=sig, MARGIN=1, v2=lim.cand[2], v3=lim.cand[1], p3)}
    if(is.null(mat4)==FALSE){m4<-apply(X=mat4, FUN=sig, MARGIN=1, v2=lim.cand[2], v3=lim.cand[1], p4)}
    
    if(sum.type=="any"){optimand<-sum(pmax(m1, m2, m3, m4, na.rm=T))/sum(n1)}
    if(sum.type=="all"){optimand<-sum(pmin(m1, m2, m3, m4, na.rm=T))/sum(n1)}
    
    return((optimand-err)^2)
  }
  
  # find the t quantile limits that get as close as possible to a 5% rejection rate
  # in the bootstrap sample at hand
  
  if(type=="any"){out<-optimize(f=lims, interval=c(0.001, 5), sum.type=type)}
  if(type=="all"){out<-optimize(f=lims, interval=c(0.001, 2), sum.type=type)}
  
  # if there were no hits at all, just return t=0 as the optimum
  if(err^2==out$objective){out<-list(minimum=0, objective=NA); return(out)}else{return(out)}
  
}


# function used to create non-parametrically bootstrapped samples of marginal effects
# calculated from a model with interaction; takes data, model, an index of observations, 
# and identities of the interacted variables and returns a vector of t-statistics
# corresponding to a single bootstrap deviate (of observations indexed in K) for 
# marginal effects in the direction of x.name and z.name
boot.fun<-function(dat, K, form, fam="gaussian", x.name, z.name){
  
  # dat: the data set on which the original model was run
  # K: index of observations in dat that will be selected for a particular bootstrap run
  # form: the formula for the GLM model
  # fam: the family for the GLM model
  # x.name: the identity of the first interacted variable
  # z.name: the identity of the second interacted variable
  
  # note: form should include x.name*z.name, in that order.
  
  # remove all extraneous variables from the data, delete missing observations
  dat<-subset(dat, select=all.vars(form))
  dat<-na.omit(dat)
  
  # anticipate what the name of the interaction term will be
  int.name<-paste(x.name, ":", z.name, sep="")
  
  # fit the model to the full data set
  int.mod<-glm(formula=form, family=fam, data=dat)
  
  # determine what MEs to calculate in the x dimension
  # calculate at every unique value of z, OR ten evenly spaced values of z, depending on the nature of z
  if(length(unique(dat[[z.name]]))<=10)
  {z.test<-sort(unique(dat[[z.name]]))}else{z.test<-seq(from=min(dat[[z.name]]), to=max(dat[[z.name]]), length.out=10)}
  # calculate dy/dx | z for the values of z.test
  coef.x.o<-summary(int.mod)$coefficients[x.name,1]+summary(int.mod)$coefficients[int.name,1]*z.test
  
  # determine what MEs to calculate in the z dimension
  # calculate at every unique value of x, OR ten evenly spaced values of x, depending on the nature of x
  if(length(unique(dat[[x.name]]))<=10)
  {x.test<-sort(unique(dat[[x.name]]))}else{x.test<-seq(from=min(dat[[x.name]]), to=max(dat[[x.name]]), length.out=10)}
  # calculate dy/dz | x for the values of z.test
  coef.z.o<-summary(int.mod)$coefficients[z.name,1]+summary(int.mod)$coefficients[int.name,1]*x.test
  
  # create the bootstrapped data set using the indices K
  dat.boot<-dat[K,]
  
  # run the model on the bootstrapped data
  int.mod.store<-glm(formula=form, family=fam, data=dat.boot)
  
  # calculate dy/dx | z for the values of z.test
  coef.x<-summary(int.mod.store)$coefficients[x.name,1]+summary(int.mod.store)$coefficients[int.name,1]*z.test
  se.x<-sqrt(vcov(int.mod.store)[x.name,x.name]+(z.test^2)*vcov(int.mod.store)[int.name,int.name]+2*z.test*vcov(int.mod.store)[x.name,int.name])
  
  # calculate the bootstrap-t values for dy/dx | z
  t.stat.x<-(coef.x-coef.x.o)/se.x
  
  # calculate dy/dz | x for the values of z.test
  coef.z<-summary(int.mod.store)$coefficients[z.name,1]+summary(int.mod.store)$coefficients[int.name,1]*x.test
  se.z<-sqrt(vcov(int.mod.store)[z.name,z.name]+(x.test^2)*vcov(int.mod.store)[int.name,int.name]+2*x.test*vcov(int.mod.store)[z.name,int.name])
  
  # calculate the bootstrap-t values for dy/dz | x
  t.stat.z<-(coef.z-coef.z.o)/se.z
  
  # return the bootstrap-t statistics from the function
  return(c(t.stat.x, t.stat.z))
  
}






dat<-read.dta(file="coxappend.dta")

dat$uppereneth <- dat$upper*dat$eneth
dat<-subset(dat, drop==0)


set.seed(1231124)


library(boot)
boot.t.dist <- boot(data = dat, statistic = boot.fun, R = 10000, form=enpv ~ proximit * enpres + eneth + lnml + lmleneth + uppereneth + upper, fam="gaussian", x.name="proximit", z.name="enpres")$t
boot.t.x.dist<-boot.t.dist[,1:10]
find.multi.lims(boot.t.x.dist, type="any", err=0.1)$minimum         # answer:  3.424815




dat$proxpres <- dat$proximit*dat$enpres


set.seed(1231124)

library(boot)
boot.t.dist <- boot(data = dat, statistic = boot.fun, R = 10000, form=enpv ~ eneth * lnml + uppereneth + upper + enpres + proximit + proxpres, fam="gaussian", x.name="eneth", z.name="lnml")$t
boot.t.x.dist<-boot.t.dist[,1:10]
find.multi.lims(boot.t.x.dist, type="any", err=0.1)$minimum         # answer: 2.884529

# this is the calculation for prediction-corrected t-statistics for the Amorim Neto and Cox data
boot.t.x.dist.lo<-boot.t.dist[,1:5]
boot.t.x.dist.hi<-boot.t.dist[,6:10]
find.multi.lims(boot.t.x.dist.lo, boot.t.x.dist.hi, type="all", p1=0, p2=1, err=0.1)$minimum     # answer: 1.464103






dat<-read.dta(file="legislative_figure.dta")
dat<-subset(dat, subset=(nineties==1 & old==1))

set.seed(1231124)

library(boot)
boot.t.dist <- boot(data = dat, statistic = boot.fun, R = 10000, form=enep1 ~ eneg * logmag + uppertier_eneg + uppertier + proximity1 + proximity1_enpres + enpres, fam="gaussian", x.name="eneg", z.name="logmag")$t
boot.t.x.dist<-boot.t.dist[,1:10]
find.multi.lims(boot.t.x.dist, type="any", err=0.1)$minimum         # answer: 2.698482

# this is the calculation for prediction-corrected t-statistics for established democracies in the 1990s
boot.t.x.dist.lo<-boot.t.dist[,1:5]
boot.t.x.dist.hi<-boot.t.dist[,6:10]
find.multi.lims(boot.t.x.dist.lo, boot.t.x.dist.hi, type="all", p1=0, p2=1, err=0.1)$minimum     # answer: 1.499963







dat<-read.dta(file="legislative_figure.dta")
dat<-subset(dat, subset=(old==1))

set.seed(1231124)

library(boot)
boot.t.dist <- boot(data = dat, statistic = boot.fun, R = 10000, form=enep1 ~ eneg * logmag + uppertier_eneg + uppertier + proximity1 + proximity1_enpres + enpres, fam="gaussian", x.name="eneg", z.name="logmag")$t
boot.t.x.dist<-boot.t.dist[,1:10]
find.multi.lims(boot.t.x.dist, type="any", err=0.1)$minimum         # answer: 2.759415

# this is the calculation for prediction-corrected t-statistics for established democracies
boot.t.x.dist.lo<-boot.t.dist[,1:5]
boot.t.x.dist.hi<-boot.t.dist[,6:10]
find.multi.lims(boot.t.x.dist.lo, boot.t.x.dist.hi, type="all", p1=0, p2=1, err=0.1)$minimum     # answer: 1.554811



