#################################################
# Figure 1, Panel A: Show example ME plots for
# continuous Z case
#################################################

rm(list=ls())
set.seed(123891)
# make some data
X<-runif(50, min=-10, max=10)
Z<-runif(50, min=-10, max=10)
XZ<-X*Z
const<-rep(1, 50)
b<-c(0.15, 2.5, -2.5, -0.5)
y<-cbind(const, X, Z, XZ)%*%b+rnorm(50, mean=0, sd=15)


# save this data
require(foreign)
data<-as.data.frame(cbind(y, X, Z))
names(data)<-c("y", "X", "Z")
write.dta(data, file="interaction_cont.dta")


# analyze data from scratch
#############################

# clear memory
rm(list=ls())

# read in data
data<-read.dta(file="interaction_cont.dta")
attach(data)

# run a linear regression 
model<-lm(y~X*Z)
summary(model)

# coefficient on X*Z is sufficient to test whether X changes the relationship between y and Z (and vice versa)
# but how do we test for the statistical significance of the relationship between y and X (or y and Z)?

# recover the coefficients from the model
coef<-coefficients(model)
# recover the vcv matrix
vcv<-vcov(model)


postscript(file="sampleME.eps", height=5.5, width=5)

# plot derivative values
Z.fits<-seq(from=-10, to=10, by=0.01)
# this is the main effect
dYdX<-coef[2]+Z.fits*coef[4]
plot(dYdX~Z.fits, type="l", ylim=c(-6, 12), xlab="Z", ylab="dY / dX", main="dY / dX at different Z with 95% CIs")
legend("topright", lty=c(1,2), legend=c("marginal effect", "95% CI"))

# what is the standard error of this quantity?
# exact calculation 
# fixed the standard error here, which used the wrong formula at first
dYdX.se<-sqrt(vcv[2,2]+Z.fits^2*vcv[4,4]+2*(Z.fits)*vcv[2,4])
# 95% CI
lines(dYdX+1.96*dYdX.se~Z.fits, lty=2)
lines(dYdX-1.96*dYdX.se~Z.fits, lty=2)
# show the zero line
abline(h=0, lty=3)
#dev.copy2eps(file="sampleME.eps")
dev.off()





#################################################
# Figure 1, Panel B: Show example ME plots for
# dichotomous Z case
#################################################

rm(list=ls())
set.seed(123809)

# make some data
X<-runif(50, min=-10, max=10)
Z<-rbinom(50, size=1, prob=0.5)
XZ<-X*Z
const<-rep(1, 50)
b<-c(0.15, 2.5, -2.5, -5)
y<-cbind(const, X, Z, XZ)%*%b+rnorm(50, mean=0, sd=15)


# save this data
require(foreign)
data<-as.data.frame(cbind(y, X, Z))
names(data)<-c("y", "X", "Z")
write.dta(data, file="interaction_binary.dta")



# analyze data from scratch
#############################

# clear memory
rm(list=ls())

# read in data
data<-read.dta(file="interaction_binary.dta")
attach(data)

# run a linear regression 
model<-lm(y~X*Z)
summary(model)

# coefficient on X*Z is sufficient to test whether X changes the relationship between y and Z (and vice versa)
# but how do we test for the statistical significance of the relationship between y and X (or y and Z)?

# recover the coefficients from the model
coef<-coefficients(model)
# recover the vcv matrix
vcv<-vcov(model)

postscript(file="sampleMEdich.eps", height=5.5, width=5)

require(gplots)
me<-c(coef[2],coef[2]+coef[4])
up<-c(me[1]+1.96*sqrt(vcv[2,2]), me[2]+1.96*sqrt(vcv[2,2]+vcv[4,4]+2*vcv[2,4]))
low<-c(me[1]-1.96*sqrt(vcv[2,2]), me[2]-1.96*sqrt(vcv[2,2]+vcv[4,4]+2*vcv[2,4]))

plotCI(me, ui=up, li=low, xlim=c(0.5, 2.5), xaxt="n", ylab="dY / dX", main="dY / dX at different Z with 95% CIs", xlab="")
axis(side=1, at=1:2, labels=c("z = 0", "z = 1"), cex=0.7)
abline(h=0, lty=2)

dev.off()
#dev.copy2eps(file="sampleMEdich.eps")

# alt method
dYdX <- coef[2]+c(0,1)*coef[4]
dYdX.se<-sqrt(vcv[2,2]+c(0,1)^2*vcv[4,4]+2*c(0,1)*vcv[2,4])
plot(0:1,dYdX,ylim=c(-3.5,3.25),xlim=c(-0.5,1.5))
segments(x0=c(0:1),y0=dYdX,y1=dYdX+1.96*dYdX.se)
segments(x0=c(0:1),y0=dYdX,y1=dYdX-1.96*dYdX.se)






##########################################################
## Table 1, 2 or 3 categories, Theoretical SEs
## overconfidence in dy/dx
##########################################################

# X only interaction, 2 categories of Z
1-(1-0.05)^2                                                              # = 0.0975
# X only interaction, 3 categories of Z
1-(1-0.05)^3                                                              # = 0.142625







##########################################################
## Table 1, 2 categories, Simulated SEs
## overconfidence in dy/dx
##########################################################

rm(list=ls())                            

set.seed(134978)
require(mvtnorm)
require(cubature)

# this is a simulation for two-category discrete interactions
# with simulated confidence intervals
# testing for the FALSE POSITIVE rate 

reps<-10000
N<-1000
NSIM<-1000

pb <- txtProgressBar(min = 0, max = reps, style = 3)
noint.x.sig<-c()
x.sig<-c()
xz.sig<-c()
z.sig<-c()
int.sig<-c()
x.sig.ot<-c()
xz.sig.ot<-c()
f.test<-c()
p.val.x<-c()
p.val.xz<-c()
x.ci.true<-c()
x.ci.cor.true<-c()

for(i in 1:reps){
  
  
  setTxtProgressBar(pb, i)
  x<-runif(N)
  z<-ifelse(runif(N)>0.5,1,0)
  y<-0.2+rnorm(N, mean=0, sd=1)
  
  noint.mod.store<-lm(y~x)
  noint.x.sig[i]<-ifelse(summary(noint.mod.store)$coefficients[2,4]<0.05,1,0)
  
  
  int.mod.store<-lm(y~x*z)
  int.sig[i]<-ifelse(summary(int.mod.store)$coefficients[4,4]<0.05,1,0)
  sims<-rmvnorm(NSIM, mean=int.mod.store$coefficients, sigma=vcov(int.mod.store))
  b.x<-sims[,2]
  b.xz<-sims[,2]+sims[,4]
  b.z<-matrix(data=sims[,3]+t(t(sims[,4]))%*%t(seq(from=0, to=1, by=0.1)),ncol=11)
  x.sig[i]<-ifelse(sum(b.x>0)<(0.025*NSIM) | sum(b.x<0)<(0.025*NSIM),1,0)
  x.sig.ot[i]<-ifelse(sum(b.x>0)<(0.05*NSIM),1,0)
  xz.sig[i]<-ifelse(sum(b.xz>0)<(0.025*NSIM) | sum(b.xz<0)<(0.025*NSIM),1,0)
  xz.sig.ot[i]<-ifelse(sum(b.xz<0)<(0.05*NSIM),1,0)
  z.sig[i]<-max(ifelse(apply(X=(b.z>0),MARGIN=2,FUN=sum)<(0.025*NSIM) | apply(X=(b.z<0),MARGIN=2,FUN=sum)<(0.025*NSIM), 1, 0))
  f.test[i]<-anova(lm(y~z),lm(y~x*z))[2,6]<0.05
  p.val.x[i]<-min(sum(b.x>0), sum(b.x<0))/NSIM
  p.val.xz[i]<-min(sum(b.xz>0), sum(b.xz<0))/NSIM
  x.ci.true[i]<-quantile(b.x, 0.025)<=0 & quantile(b.x, 0.975)>=0
  x.ci.cor.true[i]<-quantile(b.x, 0.01265)<=0 & quantile(b.x, 0.98735)>=0
  
  
}
close(pb)


# Table 1, Simulated SE
# Proportion of time that dy/dx is stat. sig., two-tailed alpha = 0.05
# for z = 0 OR z = 1 OR both
sum(pmax(x.sig, xz.sig))/reps                                             # = 0.0986

# Proportion of time that dy/dx is stat. sig., two-tailed alpha = 0.05
# for z = 0 OR z = 1 OR both
# OR dy/dz is stat. sig. (two-tailed, a = 0.05) for any x \in [0,1]
sum(pmax(x.sig, xz.sig, z.sig))/reps                                      # = 0.1961


# condition inference on stat. sig. of interaction term
# this is the figure reported in the paper
sum(pmax(x.sig,xz.sig)*int.sig+(1-int.sig)*noint.x.sig)/reps              # = 0.0817


# condition inference on f test
# this is the figure listed near Table 5 
sum(pmax(x.sig,xz.sig)*f.test)/reps                                       # = 0.0487







##########################################################
## Table 1, 2 categories, Bootstrapped SEs
## overconfidence in dy/dx
##########################################################

rm(list=ls())                            

set.seed(43257340)
require(mvtnorm)
require(cubature)


# this is a simulation for two-category discrete interactions
# with bootstrapped confidence intervals
# testing for the FALSE POSITIVE rate 

reps<-10000
N<-1000
NSIM<-1000

pb <- txtProgressBar(min = 0, max = reps, style = 3)
noint.x.sig<-c()
x.sig<-c()
xz.sig<-c()
z.sig<-c()
int.sig<-c()
x.sig.ot<-c()
f.test<-c()
p.val.x<-c()
p.val.xz<-c()

for(i in 1:reps){
  
  
  setTxtProgressBar(pb, i)
  x<-runif(N)
  z<-ifelse(runif(N)>0.5,1,0)
  y<-0.2+rnorm(N, mean=0, sd=1)
  
  noint.mod.store<-lm(y~x)
  noint.x.sig[i]<-ifelse(summary(noint.mod.store)$coefficients[2,4]<0.05,1,0)
   
  int.mod.store<-lm(y~x*z)
  int.sig[i]<-ifelse(summary(int.mod.store)$coefficients[4,4]<0.05,1,0)
  
  sims<-matrix(data=NA, nrow=NSIM, ncol=4)
  
  #calculate bootstrapped standard errors
  for(j in 1:NSIM){
    
    boot<-sample(1:length(x), size=length(x), replace=T)
    xx<-x[boot]
    zz<-z[boot]
    yy<-y[boot]
    
    sims[j,]<-coef(lm(yy~xx*zz))
    
  }
  
  b.x<-sims[,2]
  b.xz<-sims[,2]+sims[,4]
  b.z<-matrix(data=sims[,3]+t(t(sims[,4]))%*%t(seq(from=0, to=1, by=0.1)),ncol=11)
  x.sig[i]<-ifelse(sum(b.x>0)<(0.025*NSIM) | sum(b.x<0)<(0.025*NSIM),1,0)
  x.sig.ot[i]<-ifelse(sum(b.x>0)<(0.05*NSIM),1,0)
  xz.sig[i]<-ifelse(sum(b.xz>0)<(0.025*NSIM) | sum(b.xz<0)<(0.025*NSIM),1,0)
  z.sig[i]<-max(ifelse(apply(X=(b.z>0),MARGIN=2,FUN=sum)<(0.025*NSIM) | apply(X=(b.z<0),MARGIN=2,FUN=sum)<(0.025*NSIM), 1, 0))
  f.test[i]<-anova(lm(y~1),lm(y~x*z))[2,6]<0.05
  p.val.x[i]<-min(sum(b.x>0), sum(b.x<0))/NSIM
  p.val.xz[i]<-min(sum(b.xz>0), sum(b.xz<0))/NSIM
  
}
close(pb)


# Table 1, Bootstrap SE
# Proportion of time that dy/dx is stat. sig., two-tailed alpha = 0.05
# for z = 0 OR z = 1 OR both
sum(pmax(x.sig, xz.sig))/reps                                            #  = 0.1033


# Proportion of time that dy/dx is stat. sig., two-tailed alpha = 0.05
# for z = 0 OR z = 1 OR both
# OR dy/dz is stat. sig. (two-tailed, a = 0.05) for any x \in [0,1]
sum(pmax(x.sig, xz.sig, z.sig))/reps                                     #  = 0.197


# condition inference on stat. sig. of interaction term
sum(pmax(x.sig,xz.sig)*int.sig+(1-int.sig)*noint.x.sig)/reps             #  = 0.0831


# condition inference on f test
sum(pmax(x.sig,xz.sig)*f.test)/reps                                      #  = 0.0348






##########################################################
## Table 1, 2 categories, Analytical SEs
## overconfidence in dy/dx
##
## and
## 
## Table 3, 2 Categories, Analytical SEs
## underconfidence in directional predictions of dy/dx
##########################################################


rm(list=ls())

source("2015-8-20_fdr-interaction.R")

set.seed(024823)
require(mvtnorm)
require(cubature)

# this is a simulation for a dichotomous two-category interaction
# with analytically calculated confidence intervals
reps<-10000
N<-1000


pb <- txtProgressBar(min = 0, max = reps, style = 3)
noint.x.sig<-c()
xz.sig<-matrix(data=NA, ncol=2, nrow=reps)
zx.sig<-matrix(data=NA, ncol=11, nrow=reps)
int.sig<-c()
t.mat.x.store<-matrix(data=NA, ncol=2, nrow=reps)
t.mat.z.store<-matrix(data=NA, ncol=11, nrow=reps)
xz.fdr.sig<-matrix(data=NA, ncol=2, nrow=reps)
zx.fdr.sig<-matrix(data=NA, ncol=11, nrow=reps)
all.fdr.sig<-matrix(data=NA, ncol=13, nrow=reps) 

for(i in 1:reps){
  
  
  setTxtProgressBar(pb, i)
  x<-runif(N)
  z<-ifelse(runif(N)>0.5,1,0)
  y<-0.2+rnorm(N, mean=0, sd=1)
  
  z.test<-c(0,1)
  x.test<-seq(from=0, to=1, by=0.1)
  int.mod.store<-lm(y~x*z)
  int.sig[i]<-ifelse(summary(int.mod.store)$coefficients[4,4]<0.05,1,0)
  coef.x<-summary(int.mod.store)$coefficients[2,1]+summary(int.mod.store)$coefficients[4,1]*z.test
  se.x<-sqrt(vcov(int.mod.store)[2,2]+(z.test^2)*vcov(int.mod.store)[4,4]+2*z.test*vcov(int.mod.store)[2,4])
  bound<-coef.x+1.96*sign(coef.x)*(-1)*se.x
  xz.sig[i,]<-ifelse(sign(bound)==sign(coef.x),1,0)
  t.fdr<-fdr.interaction(coef.x, se.x, df=int.mod.store$df)
  bound<-coef.x+t.fdr*sign(coef.x)*(-1)*se.x
  xz.fdr.sig[i,]<-ifelse(sign(bound)==sign(coef.x),1,0)
  
  coef.z<-summary(int.mod.store)$coefficients[3,1]+summary(int.mod.store)$coefficients[4,1]*x.test
  se.z<-sqrt(vcov(int.mod.store)[3,3]+(x.test^2)*vcov(int.mod.store)[4,4]+2*x.test*vcov(int.mod.store)[3,4])
  bound<-coef.z+1.96*sign(coef.z)*(-1)*se.z
  zx.sig[i,]<-ifelse(sign(bound)==sign(coef.z),1,0)
  t.fdr<-fdr.interaction(coef.z, se.z, df=int.mod.store$df)
  bound<-coef.z+t.fdr*sign(coef.z)*(-1)*se.z
  zx.fdr.sig[i,]<-ifelse(sign(bound)==sign(coef.z),1,0)
  
  t.stat.x<-coef.x/se.x
  t.mat.x.store[i,]<-t.stat.x
  
  t.stat.z<-coef.z/se.z
  t.mat.z.store[i,]<-t.stat.z
  
  coef.all <- c(coef.x, coef.z)
  se.all <- c(se.x, se.z)
  t.fdr<-fdr.interaction(coef.all, se.all, df=int.mod.store$df)
  bound<-coef.all+t.fdr*sign(coef.all)*(-1)*se.all
  all.fdr.sig[i,]<-ifelse(sign(bound)==sign(coef.all),1,0)
  
}
close(pb)



# Table 1, Analytical SE
# Proportion of time that dy/dx is stat. sig., two-tailed alpha = 0.05
# for z = 0 OR z = 1 OR both
sum(apply(X=xz.sig, MARGIN=1, FUN=max))/reps                                    #  = 0.0945


# Proportion of time that dy/dx is stat. sig., two-tailed alpha = 0.05
# for z = 0 OR z = 1 OR both
# OR dy/dz is stat. sig. (two-tailed, a = 0.05) for any x \in [0,1]
sum(apply(X=cbind(xz.sig, zx.sig), MARGIN=1, FUN=max))/reps                     #  = 0.1907



# Table 3, binary Z, type I error
# dy/dx = 0 when z = 0, dy/dx < 0 when z = 1
t.mat.x.match<-cbind(abs(t.mat.x.store[,1]) < 1.96, t.mat.x.store[,2] < -1.96)
sum(apply(X=t.mat.x.match, MARGIN=1, FUN=min))/reps                             #  = 0.0225



# Table 3, binary Z, type I error
# dy/dx > 0 when z = 0, dy/dx < 0 when z = 1
t.mat.x.match<-cbind(t.mat.x.store[,1] > 1.96, t.mat.x.store[,2] < -1.96)
sum(apply(X=t.mat.x.match, MARGIN=1, FUN=min))/reps                             #  = 7e-04


# Proportion of time that dy/dx is stat. sig. using FDR, two-tailed alpha = 0.05
# for z = 0 OR z = 1 OR both
sum(apply(X=xz.fdr.sig, MARGIN=1, FUN=max))/reps                                  #  = 0.048


# Proportion of time that dy/dz is stat. sig. (two-tailed, a = 0.05) 
# for any x \in [0,1]
sum(apply(X=zx.fdr.sig, MARGIN=1, FUN=max))/reps                                  #  = 0.0329


# Proportion of time that dy/dx is stat. sig. using FDR, two-tailed alpha = 0.05
# for z = 0 OR z = 1 OR both
# OR dy/dz is stat. sig. (two-tailed, a = 0.05) for any x \in [0,1]
sum(apply(X=all.fdr.sig, MARGIN=1, FUN=max))/reps                                  #  = 0.0348








##########################################################
## Table 1, 3 categories, Simulated SEs
## overconfidence in dy/dx
##########################################################



# this is a simulation for three-category discrete interactions
# with simulated confidence intervals
rm(list=ls())                            

set.seed(928483)
require(mvtnorm)
require(cubature)

reps<-10000
N<-1000
NSIM<-1000

pb <- txtProgressBar(min = 0, max = reps, style = 3)
x.sig<-c()
xz1.sig<-c()
z1.sig<-c()
xz2.sig<-c()
z2.sig<-c()


for(i in 1:reps){
  
  
  setTxtProgressBar(pb, i)
  x<-runif(N)
  z<-ifelse(runif(N)<=(1/3),0,ifelse(runif(N)<=(1/2),1,2))
  z<-as.factor(z)
  y<-0.2+0.0*x+rnorm(N, mean=0, sd=1)
  
  
  int.mod.store<-lm(y~x*z)
  sims<-rmvnorm(NSIM, mean=int.mod.store$coefficients, sigma=vcov(int.mod.store))
  b.x<-sims[,2]
  b.xz1<-sims[,2]+sims[,5]
  b.xz2<-sims[,2]+sims[,6]
  b.z1<-matrix(data=sims[,3]+t(t(sims[,5]))%*%t(seq(from=0, to=1, by=0.1)),ncol=11)
  b.z2<-matrix(data=sims[,4]+t(t(sims[,6]))%*%t(seq(from=0, to=1, by=0.1)),ncol=11)
  x.sig[i]<-ifelse(sum(b.x>0)<(0.025*NSIM) | sum(b.x<0)<(0.025*NSIM),1,0)
  xz1.sig[i]<-ifelse(sum(b.xz1>0)<(0.025*NSIM) | sum(b.xz1<0)<(0.025*NSIM),1,0)
  xz2.sig[i]<-ifelse(sum(b.xz2>0)<(0.025*NSIM) | sum(b.xz2<0)<(0.025*NSIM),1,0)
  z1.sig[i]<-max(ifelse(apply(X=(b.z1>0),MARGIN=2,FUN=sum)<(0.025*NSIM) | apply(X=(b.z1<0),MARGIN=2,FUN=sum)<(0.025*NSIM), 1, 0))
  z2.sig[i]<-max(ifelse(apply(X=(b.z2>0),MARGIN=2,FUN=sum)<(0.025*NSIM) | apply(X=(b.z2<0),MARGIN=2,FUN=sum)<(0.025*NSIM), 1, 0))
  
}
close(pb)


# Table 1, Simulated SE
# Proportion of time that dy/dx is stat. sig., two-tailed alpha = 0.05
# for any value of z \in {0, 1, 2}
sum(pmax(x.sig, xz1.sig, xz2.sig))/reps                                    #  = 0.142


# Proportion of time that dy/dx is stat. sig., two-tailed alpha = 0.05
# for any value of z \in {0, 1, 2}
# OR dy/dz is stat. sig. (two-tailed, a = 0.05) for any x \in [0,1]
sum(pmax(x.sig, xz1.sig, xz2.sig, z1.sig, z2.sig))/reps                    # = 0.3099













##########################################################
## Table 1, 3 categories, Analytic SEs
## overconfidence in dy/dx
##########################################################



rm(list=ls())

source("2015-8-20_fdr-interaction.R")

set.seed(9284728)
require(mvtnorm)
require(cubature)

# this is a simulation for a dichotmous three-category interaction
# with analytically calculated confidence intervals
reps<-10000
N<-1000

pb <- txtProgressBar(min = 0, max = reps, style = 3)
noint.x.sig<-c()
xz.sig<-matrix(data=NA, ncol=3, nrow=reps)
zx1.sig<-matrix(data=NA, ncol=11, nrow=reps)
zx2.sig<-matrix(data=NA, ncol=11, nrow=reps)
xz.fdr.sig<-matrix(data=NA, ncol=3, nrow=reps)
zx.fdr.sig<-matrix(data=NA, ncol=22, nrow=reps)
all.fdr.sig<-matrix(data=NA, ncol=25, nrow=reps) 

for(i in 1:reps){
  
  
  setTxtProgressBar(pb, i)
  x<-runif(N)
  z<-ifelse(runif(N)<=(1/3),0,ifelse(runif(N)<=(1/2),1,2))
  z<-as.factor(z)
  y<-0.2+rnorm(N, mean=0, sd=1)
  
  
  int.mod.store<-lm(y~x*z)
  coef.x<-summary(int.mod.store)$coefficients[2,1]
  se.x<-sqrt(vcov(int.mod.store)[2,2])
  bound<-coef.x+1.96*sign(coef.x)*(-1)*se.x
  xz.sig[i,1]<-ifelse(sign(bound)==sign(coef.x),1,0)

  coef.x.2<-summary(int.mod.store)$coefficients[2,1]+summary(int.mod.store)$coefficients[5,1]
  se.x.2<-sqrt(vcov(int.mod.store)[2,2]+(1^2)*vcov(int.mod.store)[5,5]+2*1*vcov(int.mod.store)[2,5])
  bound<-coef.x.2+1.96*sign(coef.x.2)*(-1)*se.x.2
  xz.sig[i,2]<-ifelse(sign(bound)==sign(coef.x.2),1,0)

  coef.x.3<-summary(int.mod.store)$coefficients[2,1]+summary(int.mod.store)$coefficients[6,1]
  se.x.3<-sqrt(vcov(int.mod.store)[2,2]+(1^2)*vcov(int.mod.store)[6,6]+2*1*vcov(int.mod.store)[2,6])
  bound<-coef.x.3+1.96*sign(coef.x.3)*(-1)*se.x.3
  xz.sig[i,3]<-ifelse(sign(bound)==sign(coef.x.3),1,0)
  
  coef.x.comb <- c(coef.x, coef.x.2, coef.x.3)
  se.x.comb <- c(se.x, se.x.2, se.x.3)
  t.fdr<-fdr.interaction(coef.x.comb, se.x.comb, df=int.mod.store$df)
  bound<-coef.x.comb+t.fdr*sign(coef.x.comb)*(-1)*se.x.comb
  xz.fdr.sig[i,]<-ifelse(sign(bound)==sign(coef.x.comb),1,0)
  
  
  x.test<-seq(from=0, to=1, by=0.1)
  coef.z<-summary(int.mod.store)$coefficients[3,1]+summary(int.mod.store)$coefficients[5,1]*x.test
  se.z<-sqrt(vcov(int.mod.store)[3,3]+(x.test^2)*vcov(int.mod.store)[5,5]+2*x.test*vcov(int.mod.store)[3,5])
  bound<-coef.z+1.96*sign(coef.z)*(-1)*se.z
  zx1.sig[i,]<-ifelse(sign(bound)==sign(coef.z),1,0)

  
  coef.z.2<-summary(int.mod.store)$coefficients[4,1]+summary(int.mod.store)$coefficients[6,1]*x.test
  se.z.2<-sqrt(vcov(int.mod.store)[4,4]+(x.test^2)*vcov(int.mod.store)[6,6]+2*x.test*vcov(int.mod.store)[4,6])
  bound<-coef.z.2+1.96*sign(coef.z.2)*(-1)*se.z.2
  zx2.sig[i,]<-ifelse(sign(bound)==sign(coef.z.2),1,0)
  
  coef.z.comb <- c(coef.z, coef.z.2)
  se.z.comb <- c(se.z, se.z.2)
  t.fdr<-fdr.interaction(coef.z.comb, se.z.comb, df=int.mod.store$df)
  bound<-coef.z.comb+t.fdr*sign(coef.z.comb)*(-1)*se.z.comb
  zx.fdr.sig[i,]<-ifelse(sign(bound)==sign(coef.z.comb),1,0)
  
  
  
  coef.all <- c(coef.x.comb, coef.z.comb)
  se.all <- c(se.x.comb, se.z.comb)
  t.fdr<-fdr.interaction(coef.all, se.all, df=int.mod.store$df)
  bound<-coef.all+t.fdr*sign(coef.all)*(-1)*se.all
  all.fdr.sig[i,]<-ifelse(sign(bound)==sign(coef.all),1,0)
  
}

close(pb)



# Table 1, Analytical SE
# Proportion of time that dy/dx is stat. sig., two-tailed alpha = 0.05
# for any value of z \in {0, 1, 2}
sum(apply(X=xz.sig, MARGIN=1, FUN=max))/reps                                  #  = 0.1393




# Proportion of time that dy/dx is stat. sig., two-tailed alpha = 0.05
# for any value of z \in {0, 1, 2}
# OR dy/dz is stat. sig. (two-tailed, a = 0.05) for any x \in [0,1]
sum(apply(X=cbind(xz.sig,zx1.sig,zx2.sig), MARGIN=1, FUN=max))/reps           #  = 0.2978



# Proportion of time that dy/dx is stat. sig. using FDR, two-tailed alpha = 0.05
# for any value of z \in {0, 1, 2}
sum(apply(X=xz.fdr.sig, MARGIN=1, FUN=max))/reps                                  #  = 0.052


# Proportion of time that dy/dz is stat. sig. (two-tailed, a = 0.05) 
# for any x \in [0,1]
sum(apply(X=zx.fdr.sig, MARGIN=1, FUN=max))/reps                                  #  = 0.0352


# Proportion of time that dy/dx is stat. sig. using FDR, two-tailed alpha = 0.05
# for any value of z \in {0, 1, 2}
# OR dy/dz is stat. sig. (two-tailed, a = 0.05) for any x \in [0,1]
sum(apply(X=all.fdr.sig, MARGIN=1, FUN=max))/reps                                  #  = 0.0362






##########################################################
## Table 1, continuous Z, Simulated SEs
## overconfidence in dy/dx
##########################################################

rm(list=ls())                            

set.seed(459873)
require(mvtnorm)
require(cubature)


# this is a simulation for a continuous interaction
# with simulated confidence intervals
reps<-10000
N<-1000
NSIM<-1000

pb <- txtProgressBar(min = 0, max = reps, style = 3)
noint.x.sig<-c()
xz.sig<-matrix(data=NA, ncol=11, nrow=reps)
zx.sig<-matrix(data=NA, ncol=11, nrow=reps)
int.sig<-c()
deflate<-c()
p.val.xz<-matrix(data=NA, ncol=11, nrow=reps)
p.val.zx<-matrix(data=NA, ncol=11, nrow=reps)
f.test<-c()

for(i in 1:reps){
  
  
  setTxtProgressBar(pb, i)
  x<-runif(N)
  z<-runif(N)
  y<-0.2+0.0*x+rnorm(N, mean=0, sd=1)
  
  noint.mod.store<-lm(y~x)
  noint.x.sig[i]<-ifelse(summary(noint.mod.store)$coefficients[2,4]<0.05,1,0)
  
  z.test<-seq(from=0, to=1, by=0.1)
  int.mod.store<-lm(y~x*z)
  int.sig[i]<-ifelse(summary(int.mod.store)$coefficients[4,4]<0.05,1,0)
  sims<-rmvnorm(NSIM, mean=int.mod.store$coefficients, sigma=vcov(int.mod.store))
  z.add<-t(t(sims[,4]))%*%t(z.test)
  b.xz<-matrix(rep(sims[,2],length(z.test)),ncol=length(z.test), nrow=NSIM,byrow=F)+z.add
  xz.sig[i,]<-ifelse(apply(X=b.xz>0, MARGIN=2, FUN=sum)<(0.025*NSIM) | apply(X=b.xz<0, MARGIN=2, FUN=sum)<(0.025*NSIM),1,0)
  p.val.xz.t1<-apply(X=b.xz>0, MARGIN=2, FUN=sum)/NSIM
  p.val.xz.t2<-apply(X=b.xz<0, MARGIN=2, FUN=sum)/NSIM
  p.val.xz[i,]<-pmin(p.val.xz.t1,p.val.xz.t2)
  
  x.test<-seq(from=0, to=1, by=0.1)
  x.add<-t(t(sims[,4]))%*%t(x.test)
  b.zx<-matrix(rep(sims[,3],length(x.test)),ncol=length(x.test), nrow=NSIM,byrow=F)+x.add
  zx.sig[i,]<-ifelse(apply(X=b.zx>0, MARGIN=2, FUN=sum)<(0.025*NSIM) | apply(X=b.zx<0, MARGIN=2, FUN=sum)<(0.025*NSIM),1,0)
  p.val.zx.t1<-apply(X=b.zx>0, MARGIN=2, FUN=sum)/NSIM
  p.val.zx.t2<-apply(X=b.zx<0, MARGIN=2, FUN=sum)/NSIM
  p.val.zx[i,]<-pmin(p.val.zx.t1,p.val.zx.t2)
  
  f.test[i]<-anova(lm(y~z),lm(y~x*z))[2,6]<0.05
  
}

close(pb)



# Table 1, Simulated SE
# Proportion of time that dy/dx is stat. sig., two-tailed alpha = 0.05
# for any value of z \in [0,1]
sum(apply(X=xz.sig, MARGIN=1, FUN=max))/reps                                             #  = 0.1451



# Proportion of time that dy/dx is stat. sig., two-tailed alpha = 0.05
# for any value of z \in [0,1]
# OR dy/dz is stat. sig. (two-tailed, a = 0.05) for any x \in [0,1]
sum(apply(X=cbind(zx.sig,xz.sig), MARGIN=1, FUN=max))/reps                               #  = 0.2261



# condition inference on stat. sig. of interaction term
sum(apply(X=xz.sig, MARGIN=1, FUN=max)*int.sig+(1-int.sig)*noint.x.sig)/reps             #  = 0.096



# condition inference on f-test
# this is the figure listed near Table 5 
sum(apply(X=xz.sig, MARGIN=1, FUN=max)*f.test)/reps                                      #  = 0.0503













##########################################################
## Table 1, continuous Z, Analytic SEs
## overconfidence in dy/dx
##
## and
## 
## Table 3, continuous, Analytical SEs
## underconfidence in directional predictions of dy/dx
##########################################################


rm(list=ls())

source("2015-8-20_fdr-interaction.R")

set.seed(1039472)
require(mvtnorm)
require(cubature)

# this is a simulation for a continuous interaction
# with analytically calculated confidence intervals
reps<-10000
N<-1000

pb <- txtProgressBar(min = 0, max = reps, style = 3)
noint.x.sig<-c()
xz.sig<-matrix(data=NA, ncol=11, nrow=reps)
zx.sig<-matrix(data=NA, ncol=11, nrow=reps)
xz.sig.fdr<-matrix(data=NA, ncol=11, nrow=reps)
zx.sig.fdr<-matrix(data=NA, ncol=22, nrow=reps)
int.sig<-c()
t.mat.x.store<-matrix(data=NA, ncol=11, nrow=reps)
t.mat.z.store<-matrix(data=NA, ncol=11, nrow=reps)

for(i in 1:reps){
  
  
  setTxtProgressBar(pb, i)
  x<-runif(N)
  z<-runif(N)
  y<-0.2+rnorm(N, mean=0, sd=1)
  
  z.test<-seq(from=0, to=1, by=0.1)
  x.test<-seq(from=0, to=1, by=0.1)
  int.mod.store<-lm(y~x*z)
  int.sig[i]<-ifelse(summary(int.mod.store)$coefficients[4,4]<0.05,1,0)
  coef.x<-summary(int.mod.store)$coefficients[2,1]+summary(int.mod.store)$coefficients[4,1]*z.test
  se.x<-sqrt(vcov(int.mod.store)[2,2]+(z.test^2)*vcov(int.mod.store)[4,4]+2*z.test*vcov(int.mod.store)[2,4])
  bound<-coef.x+1.96*sign(coef.x)*(-1)*se.x
  xz.sig[i,]<-ifelse(sign(bound)==sign(coef.x),1,0)
  fdr.t<-fdr.interaction(coef.x, se.x, df=int.mod.store$df)
  bound.fdr<-coef.x+fdr.t*sign(coef.x)*(-1)*se.x
  xz.sig.fdr[i,]<-ifelse(sign(bound.fdr)==sign(coef.x),1,0)
  
  coef.z<-summary(int.mod.store)$coefficients[3,1]+summary(int.mod.store)$coefficients[4,1]*x.test
  se.z<-sqrt(vcov(int.mod.store)[3,3]+(x.test^2)*vcov(int.mod.store)[4,4]+2*x.test*vcov(int.mod.store)[3,4])
  bound<-coef.z+1.96*sign(coef.z)*(-1)*se.z
  zx.sig[i,]<-ifelse(sign(bound)==sign(coef.z),1,0)
  
  # combined test with FDR
  fdr.t<-fdr.interaction(c(coef.z, coef.x), c(se.z, se.x), df=int.mod.store$df)
  bound.fdr<-c( coef.z+fdr.t*sign(coef.z)*(-1)*se.z, coef.x+fdr.t*sign(coef.x)*(-1)*se.x )
  zx.sig.fdr[i,]<-ifelse(sign(bound.fdr)==sign(c(coef.z, coef.x)),1,0)
  
    
  t.stat.x<-coef.x/se.x
  t.mat.x.store[i,]<-t.stat.x
  
  t.stat.z<-coef.z/se.z
  t.mat.z.store[i,]<-t.stat.z
  
}

close(pb)


# Table 1, Analytic SE
# Proportion of time that dy/dx is stat. sig., two-tailed alpha = 0.05
# for any value of z \in [0,1]
sum(apply(X=xz.sig, MARGIN=1, FUN=max))/reps                                             #  = 0.1375


# Continous, Simulated SE
sum(apply(X=cbind(xz.sig,zx.sig), MARGIN=1, FUN=max))/reps                               #  = 0.2166


# Table 3, continuous Z, type I error
# dy/dx = 0 when z < 0.5, dy/dx < 0 when z >= 0.5
t.mat.x.match<-cbind(apply(X=(abs(t.mat.x.store[,1:5]) < 1.96), MARGIN=1, FUN=min), apply(X=(t.mat.x.store[,6:11] < -1.96), FUN=max, MARGIN=1))
sum(apply(X=t.mat.x.match, MARGIN=1, FUN=min))/reps
#  Answer: 0.0281

# Table 3, continuous Z, type I error
# dy/dx > 0 when z < 0.5, dy/dx < 0 when z >= 0.5
t.mat.x.match<-cbind(apply(X=((t.mat.x.store[,1:5]) > 1.96), MARGIN=1, FUN=max), apply(X=(t.mat.x.store[,6:11] < -1.96), FUN=max, MARGIN=1))
sum(apply(X=t.mat.x.match, MARGIN=1, FUN=min))/reps
#  Answer: 0.0049

# Table 3, continuous Z, type I error
# dy/dx > 0 when z < 0.5, dy/dx < 0 when z >= 0.5
# dy/dz > 0 when x < 0.5, dy/dz < 0 when x >= 0.5
t.mat.both.match<-cbind(apply(X=((t.mat.x.store[,1:5]) > 1.96), MARGIN=1, FUN=max), apply(X=(t.mat.x.store[,6:11] < -1.96), FUN=max, MARGIN=1), apply(X=((t.mat.z.store[,1:5]) > 1.96), MARGIN=1, FUN=max), apply(X=((t.mat.z.store[,6:11]) < -1.96), MARGIN=1, FUN=max))
sum(apply(X=t.mat.both.match, MARGIN=1, FUN=min))/reps
#  Answer: 0.0034

# Table 3, continuous Z, type I error
# dy/dx > 0 when z < 0.5, dy/dx < 0 when z >= 0.5
# dy/dz < 0 for all x
t.mat.both.match<-cbind(apply(X=((t.mat.x.store[,1:5]) > 1.96), MARGIN=1, FUN=max), apply(X=(t.mat.x.store[,6:11] < -1.96), FUN=max, MARGIN=1), apply(X=((t.mat.z.store[,1:11]) < -1.96), MARGIN=1, FUN=max))
sum(apply(X=t.mat.both.match, MARGIN=1, FUN=min))/reps
#  Answer: 0.004

# Table 3, continuous Z, type I error
# dy/dx > 0 when z < 0.5, dy/dx < 0 when z >= 0.5
# dy/dz < 0 when x < 0.5, dy/dz > 0 when x >= 0.5
t.mat.both.match<-cbind(apply(X=((t.mat.x.store[,1:5]) > 1.96), MARGIN=1, FUN=max), apply(X=(t.mat.x.store[,6:11] < -1.96), FUN=max, MARGIN=1), apply(X=((t.mat.z.store[,1:5]) < -1.96), MARGIN=1, FUN=max), apply(X=((t.mat.z.store[,6:11]) > 1.96), MARGIN=1, FUN=max))
sum(apply(X=t.mat.both.match, MARGIN=1, FUN=min))/reps
#  Answer: 0


# Proportion of time that dy/dx is stat. sig. using FDR, two-tailed alpha = 0.05
# for any value of z \in [0,1]
sum(apply(X=xz.sig.fdr, MARGIN=1, FUN=max))/reps      # 0.034

# Proportion of time that dy/dx and dy/dz is stat. sig. using FDR, two-tailed alpha = 0.05
# for any value of z \in [0,1]
sum(apply(X=xz.sig.fdr, MARGIN=1, FUN=max))/reps      # 0.031





##########################################################
## Table 2, binary Z, Analytical SEs
## overconfidence in predictions of dy/dx
## 
## and
##
## Table 4, binary z, FDR and FWER corrected
## tests for significance of dy/dx
##########################################################

rm(list=ls())    

source("2015-8-20_fdr-interaction.R")

set.seed(024823)
require(mvtnorm)
require(cubature)

#################################################################
# this is a simulation for a dichotomous two-category interaction
# with analytically calculated confidence intervals
# we allow the correlation between X and Z to vary to change
# the false positive rate
#################################################################


reps<-10000
N<-1000

R<-c(0.99, 0.9, 0.5, 0.2, 0, -0.2, -0.5, -0.9, -0.99)

dydx.sig<-c()
dydz.sig<-c()
dydx.fdr.sig<-c()
dydz.fdr.sig<-c()
both.sig<-c()
t.corr<-c()
dydx.f.sig<-c()

for(j in 1:length(R)){
  
  cat("Correlation = ", R[j], "\n", sep="")
  noint.x.sig<-c()
  xz.sig<-matrix(data=NA, ncol=2, nrow=reps)
  zx.sig<-matrix(data=NA, ncol=11, nrow=reps)
  xz.sig.fdr<-matrix(data=NA, ncol=2, nrow=reps)
  zx.sig.fdr<-matrix(data=NA, ncol=11, nrow=reps)
  int.sig<-c()
  t.mat.x.store<-matrix(data=NA, ncol=2, nrow=reps)
  t.mat.z.store<-matrix(data=NA, ncol=11, nrow=reps)
  f.test <- c()
  
  SIGMA <- matrix(data=c(1, R[j], R[j], 1), nrow=2, ncol=2)
  
  pb <- txtProgressBar(min = 0, max = reps, style = 3)
  
  for(i in 1:reps){
    
    setTxtProgressBar(pb, i)
    
    X<-rmvnorm(N, mean=c(0,0), sigma=SIGMA)
    x<-X[,1]
    z<-runif(N)<pnorm(X[,2], mean=0, sd=0.5)
    y<-0.2+rnorm(N, mean=0, sd=1)
    
    z.test<-c(0,1)
    x.test<-seq(from=0, to=1, by=0.1)
    int.mod.store<-lm(y~x*z)
    int.sig[i]<-ifelse(summary(int.mod.store)$coefficients[4,4]<0.05,1,0)
    coef.x<-summary(int.mod.store)$coefficients[2,1]+summary(int.mod.store)$coefficients[4,1]*z.test
    se.x<-sqrt(vcov(int.mod.store)[2,2]+(z.test^2)*vcov(int.mod.store)[4,4]+2*z.test*vcov(int.mod.store)[2,4])
    bound<-coef.x+1.96*sign(coef.x)*(-1)*se.x
    xz.sig[i,]<-ifelse(sign(bound)==sign(coef.x),1,0)
    fdr.t<-fdr.interaction(coef.x, se.x, df=int.mod.store$df)
    bound.fdr<-coef.x+fdr.t*sign(coef.x)*(-1)*se.x
    xz.sig.fdr[i,]<-ifelse(sign(bound.fdr)==sign(coef.x),1,0)
    
    coef.z<-summary(int.mod.store)$coefficients[3,1]+summary(int.mod.store)$coefficients[4,1]*x.test
    se.z<-sqrt(vcov(int.mod.store)[3,3]+(x.test^2)*vcov(int.mod.store)[4,4]+2*x.test*vcov(int.mod.store)[3,4])
    bound<-coef.z+1.96*sign(coef.z)*(-1)*se.z
    zx.sig[i,]<-ifelse(sign(bound)==sign(coef.z),1,0)
    fdr.t<-fdr.interaction(coef.z, se.z, df=int.mod.store$df)
    bound.fdr<-coef.z+fdr.t*sign(coef.z)*(-1)*se.z
    zx.sig.fdr[i,]<-ifelse(sign(bound.fdr)==sign(coef.z),1,0)
    
    t.stat.x<-coef.x/se.x
    t.mat.x.store[i,]<-t.stat.x
    
    t.stat.z<-coef.z/se.z
    t.mat.z.store[i,]<-t.stat.z
    
    f.test[i]<-anova(lm(y~z),lm(y~x*z))[2,6]<0.05
    
  }
  close(pb)
  
  #Table 1, 2 categories, Analytical SEs
  dydx.sig[j]<-sum(apply(X=xz.sig, MARGIN=1, FUN=max))/reps
  dydz.sig[j]<-sum(apply(X=zx.sig, MARGIN=1, FUN=max))/reps
  #Table 3, 2 categories, Analytical SEs
  both.sig[j]<-sum(apply(X=cbind(xz.sig,zx.sig), MARGIN=1, FUN=max))/reps
  t.corr[j]<-cor(t.mat.x.store)[1,2]
  
  #2 categories, Analytical SEs, FDR boundaries
  dydx.fdr.sig[j]<-sum(apply(X=xz.sig.fdr, MARGIN=1, FUN=max))/reps
  dydz.fdr.sig[j]<-sum(apply(X=zx.sig.fdr, MARGIN=1, FUN=max))/reps
  
  #Condition on F-test
  dydx.f.sig[j]<-sum(apply(X=xz.sig, MARGIN=1, FUN=max)*f.test)/reps   
  
}

# these are the Table 2, Column 1 results
##########################################
cbind(R, dydx.sig)

# results
#          R dydx.sig
#[1,]  0.99   0.0991
#[2,]  0.90   0.0926
#[3,]  0.50   0.0981
#[4,]  0.20   0.0978
#[5,]  0.00   0.0983
#[6,] -0.20   0.1003
#[7,] -0.50   0.1003
#[8,] -0.90   0.0975
#[9,] -0.99   0.0973



# in text information: correlation between the t-statistics
# (near Table 2)
summary(t.corr)

#       Min.    1st Qu.     Median       Mean    3rd Qu.       Max. 
# -0.0073030 -0.0053840  0.0004954  0.0019150  0.0082690  0.0140500 



# These are the Table 4, Column 1 results
# results using the FDR procedure
###########################################
cbind(R, dydx.fdr.sig)

# results

# R dydx.fdr.sig
# [1,]  0.99       0.0498
# [2,]  0.90       0.0478
# [3,]  0.50       0.0495
# [4,]  0.20       0.0513
# [5,]  0.00       0.0525
# [6,] -0.20       0.0509
# [7,] -0.50       0.0504
# [8,] -0.90       0.0502
# [9,] -0.99       0.0503


cbind(R, dydz.fdr.sig)

# results

# R dydz.fdr.sig
# [1,]  0.99       0.0313
# [2,]  0.90       0.0335
# [3,]  0.50       0.0336
# [4,]  0.20       0.0301
# [5,]  0.00       0.0305
# [6,] -0.20       0.0319
# [7,] -0.50       0.0292
# [8,] -0.90       0.0306
# [9,] -0.99       0.0337

# these are the Table 4, Column 4 results
# condition on F-test
###########################################

cbind(R, dydx.f.sig)

# R dydx.f.sig
# [1,]  0.99     0.0487
# [2,]  0.90     0.0468
# [3,]  0.50     0.0448
# [4,]  0.20     0.0476
# [5,]  0.00     0.0488
# [6,] -0.20     0.0478
# [7,] -0.50     0.0493
# [8,] -0.90     0.0481
# [9,] -0.99     0.0482


###############################################################
## Table 2, continuous uniform X + Z, Analytical SEs
## overconfidence in tests of dy/dx
## 
## and
##
## Table 4, continuous uniform X + Z, FDR and FWER corrected
## tests for significance of dy/dx
###############################################################

rm(list=ls())     

source("2015-8-20_fdr-interaction.R")

set.seed(024823)
require(mvtnorm)
require(cubature)
require(copula)

#################################################################
# this is a simulation for a continuous interaction
# with analytically calculated confidence intervals
# we allow the correlation between X and Z to vary to change
# the false positive rate
#
# uniformly distributed X and Z, with normal copula
#################################################################

reps<-10000
N<-1000

dydx.sig<-c()
dydz.sig<-c()
dydx.fdr.sig<-c()
dydz.fdr.sig<-c()
both.sig<-c()
dydx.f.sig<-c()

R<-c(0.99, 0.9, 0.5, 0.2, 0, -0.2, -0.5, -0.9, -0.99)

for(j in 1:length(R)){
  
  cat("Correlation = ", R[j], "\n", sep="")
  
  pb <- txtProgressBar(min = 0, max = reps, initial=1, style = 3)
  noint.x.sig<-c()
  xz.sig<-matrix(data=NA, ncol=11, nrow=reps)
  zx.sig<-matrix(data=NA, ncol=11, nrow=reps)
  xz.sig.fdr<-matrix(data=NA, ncol=11, nrow=reps)
  zx.sig.fdr<-matrix(data=NA, ncol=11, nrow=reps)
  int.sig<-c()
  f.test<-c()
  # SIGMA <- matrix(data=c(1, R[j], R[j], 1), nrow=2, ncol=2)
  
  for(i in 1:reps){
    
    
    setTxtProgressBar(pb, i)
    #   X<-rmvnorm(N, mean=c(0,0), sigma=SIGMA)
    #   x<-X[,1]
    #   z<-X[,2]
    X<-rCopula(N, normalCopula(R[j]))
    x<-X[,1]
    z<-X[,2]
    y<-0.2+0.0*x+rnorm(N, mean=0, sd=1)
    
    z.test<-seq(from=0, to=1, by=0.1)
    x.test<-seq(from=0, to=1, by=0.1)
    int.mod.store<-lm(y~x*z)
    int.sig[i]<-ifelse(summary(int.mod.store)$coefficients[4,4]<0.05,1,0)
    coef.x<-summary(int.mod.store)$coefficients[2,1]+summary(int.mod.store)$coefficients[4,1]*z.test
    se.x<-sqrt(vcov(int.mod.store)[2,2]+(z.test^2)*vcov(int.mod.store)[4,4]+2*z.test*vcov(int.mod.store)[2,4])
    bound<-coef.x+1.96*sign(coef.x)*(-1)*se.x
    xz.sig[i,]<-ifelse(sign(bound)==sign(coef.x),1,0)
    fdr.t<-fdr.interaction(coef.x, se.x, df=int.mod.store$df)
    bound.fdr<-coef.x+fdr.t*sign(coef.x)*(-1)*se.x
    xz.sig.fdr[i,]<-ifelse(sign(bound.fdr)==sign(coef.x),1,0)
    
    coef.z<-summary(int.mod.store)$coefficients[3,1]+summary(int.mod.store)$coefficients[4,1]*x.test
    se.z<-sqrt(vcov(int.mod.store)[3,3]+(x.test^2)*vcov(int.mod.store)[4,4]+2*x.test*vcov(int.mod.store)[3,4])
    bound<-coef.z+1.96*sign(coef.z)*(-1)*se.z
    zx.sig[i,]<-ifelse(sign(bound)==sign(coef.z),1,0)
    fdr.t<-fdr.interaction(coef.z, se.z, df=int.mod.store$df)
    bound.fdr<-coef.z+fdr.t*sign(coef.z)*(-1)*se.z
    zx.sig.fdr[i,]<-ifelse(sign(bound.fdr)==sign(coef.z),1,0)
    
    f.test[i]<-anova(lm(y~z),lm(y~x*z))[2,6]<0.05
    
  }
  
  close(pb)
  
  dydx.sig[j]<-sum(apply(X=xz.sig, MARGIN=1, FUN=max))/reps
  dydz.sig[j]<- sum(apply(X=zx.sig, MARGIN=1, FUN=max))/reps
  both.sig[j]<-sum(apply(X=cbind(xz.sig,zx.sig), MARGIN=1, FUN=max))/reps
  
  #FDR boundaries
  dydx.fdr.sig[j]<-sum(apply(X=xz.sig.fdr, MARGIN=1, FUN=max))/reps
  dydz.fdr.sig[j]<-sum(apply(X=zx.sig.fdr, MARGIN=1, FUN=max))/reps
  
  #Condition on F-test
  dydx.f.sig[j]<-sum(apply(X=xz.sig, MARGIN=1, FUN=max)*f.test)/reps  
  
}


# these are the Table 2, Column 2 results
##########################################
cbind(R, dydx.sig)


# results

#         R dydx.sig
#[1,]  0.99   0.0729
#[2,]  0.90   0.1180
#[3,]  0.50   0.1406
#[4,]  0.20   0.1382
#[5,]  0.00   0.1369
#[6,] -0.20   0.1360
#[7,] -0.50   0.1381
#[8,] -0.90   0.1157
#[9,] -0.99   0.0761


# These are the Table 4, Column 2 results
###########################################

# fdr-based results

cbind(R, dydx.fdr.sig)

# [1,]  0.99       0.0294
# [2,]  0.90       0.0319
# [3,]  0.50       0.0365
# [4,]  0.20       0.0323
# [5,]  0.00       0.0345
# [6,] -0.20       0.0320
# [7,] -0.50       0.0353
# [8,] -0.90       0.0313
# [9,] -0.99       0.0324


# These are the Table 4, Column 5 results
###########################################

# results conditional on F-test
cbind(R, dydx.f.sig)

# R dydx.f.sig
# [1,]  0.99     0.0343
# [2,]  0.90     0.0470
# [3,]  0.50     0.0538
# [4,]  0.20     0.0480
# [5,]  0.00     0.0517
# [6,] -0.20     0.0494
# [7,] -0.50     0.0531
# [8,] -0.90     0.0462
# [9,] -0.99     0.0339



##########################################################
## Tables 2 and X, continuous normal X + Z, Analytical SEs
## overconfidence in tests of dy/dx
## 
## and
##
## Table 4, continuous normal X + Z, FDR and FWER corrected
## tests for significance of dy/dx
###############################################################

rm(list=ls())       

source("2015-8-20_fdr-interaction.R")

set.seed(024823)
require(mvtnorm)
require(cubature)
require(copula)

#################################################################
# this is a simulation for a continuous interaction
# with analytically calculated confidence intervals
# we allow the correlation between X and Z to vary to change
# the false positive rate
#
# bivariate normally distributed X and Z with correlation
#################################################################

reps<-10000
N<-1000

dydx.sig<-c()
dydz.sig<-c()
dydx.fdr.sig<-c()
dydz.fdr.sig<-c()
both.sig<-c()
dydx.f.sig<-c()

R<-c(0.99, 0.9, 0.5, 0.2, 0, -0.2, -0.5, -0.9, -0.99)

for(j in 1:length(R)){
  
  cat("Correlation = ", R[j], "\n", sep="")
  
  pb <- txtProgressBar(min = 0, max = reps, initial=1, style = 3)
  noint.x.sig<-c()
  xz.sig<-matrix(data=NA, ncol=11, nrow=reps)
  zx.sig<-matrix(data=NA, ncol=11, nrow=reps)
  xz.sig.fdr<-matrix(data=NA, ncol=11, nrow=reps)
  zx.sig.fdr<-matrix(data=NA, ncol=11, nrow=reps)
  int.sig<-c()
  SIGMA <- matrix(data=c(1, R[j], R[j], 1), nrow=2, ncol=2)
  xz.f.sig<-matrix(data=NA, ncol=11, nrow=reps)
  f.test<-c()
  
  for(i in 1:reps){
    
    setTxtProgressBar(pb, i)
    
    X<-rmvnorm(N, mean=c(0,0), sigma=SIGMA)
    x<-X[,1]
    z<-X[,2]
    #   X<-rCopula(N, normalCopula(R[j]))
    #   x<-X[,1]
    #   z<-X[,2]
    y<-0.2+0.0*x+rnorm(N, mean=0, sd=1)
    
    z.test<-seq(from=0, to=1, by=0.1)
    x.test<-seq(from=0, to=1, by=0.1)
    int.mod.store<-lm(y~x*z)
    int.sig[i]<-ifelse(summary(int.mod.store)$coefficients[4,4]<0.05,1,0)
    coef.x<-summary(int.mod.store)$coefficients[2,1]+summary(int.mod.store)$coefficients[4,1]*z.test
    se.x<-sqrt(vcov(int.mod.store)[2,2]+(z.test^2)*vcov(int.mod.store)[4,4]+2*z.test*vcov(int.mod.store)[2,4])
    bound<-coef.x+1.96*sign(coef.x)*(-1)*se.x
    xz.sig[i,]<-ifelse(sign(bound)==sign(coef.x),1,0)
    fdr.t<-fdr.interaction(coef.x, se.x, df=int.mod.store$df)
    bound.fdr<-coef.x+fdr.t*sign(coef.x)*(-1)*se.x
    xz.sig.fdr[i,]<-ifelse(sign(bound.fdr)==sign(coef.x),1,0)
    
    coef.z<-summary(int.mod.store)$coefficients[3,1]+summary(int.mod.store)$coefficients[4,1]*x.test
    se.z<-sqrt(vcov(int.mod.store)[3,3]+(x.test^2)*vcov(int.mod.store)[4,4]+2*x.test*vcov(int.mod.store)[3,4])
    bound<-coef.z+1.96*sign(coef.z)*(-1)*se.z
    zx.sig[i,]<-ifelse(sign(bound)==sign(coef.z),1,0)
    fdr.t<-fdr.interaction(coef.z, se.z, df=int.mod.store$df)
    bound.fdr<-coef.z+fdr.t*sign(coef.z)*(-1)*se.z
    zx.sig.fdr[i,]<-ifelse(sign(bound.fdr)==sign(coef.z),1,0)
    
    f.test[i]<-anova(lm(y~z),lm(y~x*z))[2,6]<0.05
    
  }
  
  close(pb)
  
  dydx.sig[j]<-sum(apply(X=xz.sig, MARGIN=1, FUN=max))/reps
  dydz.sig[j]<- sum(apply(X=zx.sig, MARGIN=1, FUN=max))/reps
  both.sig[j]<-sum(apply(X=cbind(xz.sig,zx.sig), MARGIN=1, FUN=max))/reps
  
  #FDR boundaries
  dydx.fdr.sig[j]<-sum(apply(X=xz.sig.fdr, MARGIN=1, FUN=max))/reps
  dydz.fdr.sig[j]<-sum(apply(X=zx.sig.fdr, MARGIN=1, FUN=max))/reps
  
  #Condition on F-test
  dydx.f.sig[j]<-sum(apply(X=xz.sig, MARGIN=1, FUN=max)*f.test)/reps  
  
  
}


# these are the Table 2, Column 3 results
##########################################
cbind(R, dydx.sig)

# results

#         R dydx.sig
#[1,]  0.99   0.0528
#[2,]  0.90   0.0642
#[3,]  0.50   0.0842
#[4,]  0.20   0.0887
#[5,]  0.00   0.0868
#[6,] -0.20   0.0839
#[7,] -0.50   0.0822
#[8,] -0.90   0.0652
#[9,] -0.99   0.0501


# these are the Table 4, Column 3 results
##########################################

# fdr results
cbind(R, dydx.fdr.sig)

# R dydx.fdr.sig
# [1,]  0.99       0.0432
# [2,]  0.90       0.0359
# [3,]  0.50       0.0322
# [4,]  0.20       0.0290
# [5,]  0.00       0.0339
# [6,] -0.20       0.0309
# [7,] -0.50       0.0318
# [8,] -0.90       0.0344
# [9,] -0.99       0.0413



# these are the Table 4, Column 5 results
##########################################

# results contingent on F-test
cbind(R, dydx.f.sig)

# R dydx.f.sig
# [1,]  0.99     0.0277
# [2,]  0.90     0.0296
# [3,]  0.50     0.0376
# [4,]  0.20     0.0375
# [5,]  0.00     0.0396
# [6,] -0.20     0.0378
# [7,] -0.50     0.0366
# [8,] -0.90     0.0286
# [9,] -0.99     0.0226
